///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.AnimationDrivers;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.AnimationDrivers
{
	public class TranslateParameter : TranslateScalar<DummyScriptConfiguration>
	{
		public override void Validate( IFluonConfiguration fc, IAnimationSegmentConfiguration asc, IAnimationThreadConfiguration atc )
		{
			if( atc.AnimatedParameter == AnimatedConfigurationParameter.None )
				throw new InvalidOperationException( $"A fluon configuration parameter to animate must be specified." );
		}

		protected override void SetScalar( IFluonContext fx, IFluonConfiguration fcFrame, IAnimationThreadConfiguration atc, double progress )
		{
			SetConfigurationParameter( fcFrame, atc.AnimatedParameter, progress );
		}

		protected static void SetConfigurationParameter( IFluonConfiguration fc, AnimatedConfigurationParameter parameterToSet
			, double progress )
		{
			switch( parameterToSet )
			{
				case AnimatedConfigurationParameter.None:
					throw new InvalidOperationException( $"A fluon configuration parameter to animate must be specified." );
				case AnimatedConfigurationParameter.DomainOrigin_X:
					fc.DomainOrigin.SetOnAxis( 0, progress );
					return;
				case AnimatedConfigurationParameter.DomainOrigin_Y:
					fc.DomainOrigin.SetOnAxis( 1, progress );
					return;
				case AnimatedConfigurationParameter.DomainOrigin_Z:
					fc.DomainOrigin.SetOnAxis( 2, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorS1:
					fc.DomainProjectorS1 = progress;
					return;
				case AnimatedConfigurationParameter.DomainProjectorS2:
					fc.DomainProjectorS2 = progress;
					return;
				case AnimatedConfigurationParameter.DomainProjectorM1_X:
					fc.DomainProjectorM1.SetOnAxis( 0, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorM1_Y:
					fc.DomainProjectorM1.SetOnAxis( 1, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorM1_Z:
					fc.DomainProjectorM1.SetOnAxis( 2, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorM2_X:
					fc.DomainProjectorM2.SetOnAxis( 0, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorM2_Y:
					fc.DomainProjectorM2.SetOnAxis( 1, progress );
					return;
				case AnimatedConfigurationParameter.DomainProjectorM2_Z:
					fc.DomainProjectorM2.SetOnAxis( 2, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorS1:
					fc.FluonGeneratorS1 = progress;
					return;
				case AnimatedConfigurationParameter.FluonGeneratorS2:
					fc.FluonGeneratorS2 = progress;
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM1_X:
					fc.FluonGeneratorM1.SetOnAxis( 0, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM1_Y:
					fc.FluonGeneratorM1.SetOnAxis( 1, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM1_Z:
					fc.FluonGeneratorM1.SetOnAxis( 2, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM2_X:
					fc.FluonGeneratorM2.SetOnAxis( 0, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM2_Y:
					fc.FluonGeneratorM2.SetOnAxis( 1, progress );
					return;
				case AnimatedConfigurationParameter.FluonGeneratorM2_Z:
					fc.FluonGeneratorM2.SetOnAxis( 2, progress );
					return;
				case AnimatedConfigurationParameter.MdnEscaperS1:
					fc.MdnEscaperS1 = progress;
					return;
				case AnimatedConfigurationParameter.MdnEscaperS2:
					fc.MdnEscaperS2 = progress;
					return;
				case AnimatedConfigurationParameter.EscapeThreshold:
					fc.EscapeThreshold = progress;
					return;
				case AnimatedConfigurationParameter.MdnReducerS1:
					fc.MdnReducerS1 = progress;
					return;
				case AnimatedConfigurationParameter.MdnReducerS2:
					fc.MdnReducerS2 = progress;
					return;
				case AnimatedConfigurationParameter.MdnPerturbatorS1:
					fc.MdnPerturbatorS1 = progress;
					return;
				case AnimatedConfigurationParameter.MdnPerturbatorS2:
					fc.MdnPerturbatorS2 = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnInnerScale:
					fc.ReducedMdnInnerScale = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnInnerOffset:
					fc.ReducedMdnInnerOffset = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnMiddleScale:
					fc.ReducedMdnMiddleScale = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnMiddleOffset:
					fc.ReducedMdnMiddleOffset = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnOuterScale:
					fc.ReducedMdnOuterScale = progress;
					return;
				case AnimatedConfigurationParameter.ReducedMdnOuterOffset:
					fc.ReducedMdnOuterOffset = progress;
					return;
				case AnimatedConfigurationParameter.ScalarReducerS1:
					fc.ScalarReducerS1 = progress;
					return;
				case AnimatedConfigurationParameter.ScalarReducerS2:
					fc.ScalarReducerS2 = progress;
					return;
				case AnimatedConfigurationParameter.LowRangeColorReducer_S1:
					fc.LowRangeColorReducer.S1 = progress;
					return;
				case AnimatedConfigurationParameter.LowRangeColorReducer_S2:
					fc.LowRangeColorReducer.S2 = progress;
					return;
				case AnimatedConfigurationParameter.LowRangeColorReducer_Frequency:
					fc.LowRangeColorReducer.Frequency = progress;
					return;
				case AnimatedConfigurationParameter.LowRangeColorReducer_UnescapedPixelsFrequency:
					fc.LowRangeColorReducer.UnescapedPixelsFrequency = progress;
					return;
				case AnimatedConfigurationParameter.LowRangeColorReducer_UnescapedPixelsReducedColorOffset:
					fc.LowRangeColorReducer.UnescapedPixelsReducedColorOffset = progress;
					return;
				case AnimatedConfigurationParameter.HighRangeColorReducer_S1:
					fc.HighRangeColorReducer.S1 = progress;
					return;
				case AnimatedConfigurationParameter.HighRangeColorReducer_S2:
					fc.HighRangeColorReducer.S2 = progress;
					return;
				case AnimatedConfigurationParameter.HighRangeColorReducer_Frequency:
					fc.HighRangeColorReducer.Frequency = progress;
					return;
				case AnimatedConfigurationParameter.HighRangeColorReducer_UnescapedPixelsFrequency:
					fc.HighRangeColorReducer.UnescapedPixelsFrequency = progress;
					return;
				case AnimatedConfigurationParameter.HighRangeColorReducer_UnescapedPixelsReducedColorOffset:
					fc.HighRangeColorReducer.UnescapedPixelsReducedColorOffset = progress;
					return;
				case AnimatedConfigurationParameter.InfiniteRangeColorReducer_S1:
					fc.InfiniteRangeColorReducer.S1 = progress;
					return;
				case AnimatedConfigurationParameter.InfiniteRangeColorReducer_S2:
					fc.InfiniteRangeColorReducer.S2 = progress;
					return;
				case AnimatedConfigurationParameter.InfiniteRangeColorReducer_Frequency:
					fc.InfiniteRangeColorReducer.Frequency = progress;
					return;
				case AnimatedConfigurationParameter.InfiniteRangeColorReducer_UnescapedPixelsFrequency:
					fc.InfiniteRangeColorReducer.UnescapedPixelsFrequency = progress;
					return;
				case AnimatedConfigurationParameter.InfiniteRangeColorReducer_UnescapedPixelsReducedColorOffset:
					fc.InfiniteRangeColorReducer.UnescapedPixelsReducedColorOffset = progress;
					return;
				case AnimatedConfigurationParameter.ReducedColorOffset:
					fc.ReducedColorOffset = progress;
					return;
				case AnimatedConfigurationParameter.ColorExtractorS1:
					fc.ColorExtractorS1 = progress;
					return;
				case AnimatedConfigurationParameter.ColorExtractorS2:
					fc.ColorExtractorS2 = progress;
					return;
				default:
					throw new InvalidOperationException( $"Unhandled fluon configuration parameter to animate \"{parameterToSet}\"." );
			}
		}
	}
}
