///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.DomainProjectors
{
	/// <summary>
	/// "Phi" keeps the radius of each domain pixel within [-scalar...scalar]. This means that each domain pixel whose radius is
	/// higher than the scalar will have its radius replaced with the square of the scalar divided by the radius, that is, the
	/// radius is inverted.
	/// </summary>
	public class PhiOutside : DomainProjector<DummyScriptConfiguration>
	{
		public override string Description => "(x ^ s2).PhiOutside( s1 ) + m1; PhiOutside = |x| <= s1 ? (x) : (s1 ^ 2 / x)";

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			if( fc.DomainProjectorS1.IsZeroWithinEps() )
				throw new ArgumentException( $"{nameof( PhiOutside )}: DomainProjectorS1 == 0" );
		}

		public override Mdn Project( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, Mdn x )
		{
			return x.PowDnD1A( fc.DomainProjectorS2 ).PhiOutside( fc.DomainProjectorS1 ) + fc.DomainProjectorM1;
		}
	}
}
