///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using Fluogen.Dependz;

namespace Fluogen.DeployedScripts.Libraries
{
	/// <summary>
	/// https://en.wikipedia.org/wiki/List_of_colors_by_shade
	/// https://en.wikipedia.org/wiki/List_of_colors_(compact)
	/// </summary>
	public static class PaletteToneExtensions
	{
		public const int PassionHueHsl = 0;
		public const int RubyHueHsl = 100;
		public const int VermilionHueHsl = 300;
		public const int CoralHueHsl = 550;
		public const int TawnyHueHsl = 650;
		public const int CaramelHueHsl = 750;
		public const int MellowCaramelHueHsl = 750;
		public const int MellowCaramelHueShiftHsl = 200;
		public const int TangerineHueHsl = 850;
		public const int FulvousHueHsl = 950;
		public const int AmberHueHsl = 1100;
		public const int ForestHueHsl = 2800;
		public const int EmeraldHueHsl = 3600;
		public const int ViridianHueHsl = 4300; //? Tropical
		public const int TurquoiseHueHsl = 5300;
		public const int AzureHueHsl = 5700;
		public const int SapphireHueHsl = 6000;
		public const int UltramarineHueHsl = 6500;
		public const int IndigoHueHsl = 7100;
		public const int LavenderHueHsl = 7400;
		public const int PurpleHueHsl = 7750;
		public const int FuchsiaHueHsl = 8000;
		public const int AmaranthHueHsl = 8500;
		public const int CrimsonHueHsl = 9200;

		public const int PassionHueHcj = 700;
		public const int RubyHueHcj = 750;
		public const int VermilionHueHcj = 900;
		public const int CoralHueHcj = 1200;
		public const int TawnyHueHcj = 1400;
		public const int CaramelHueHcj = 1550;
		public const int MellowCaramelHueHcj = 1700;
		public const int MellowCaramelHueShiftHcj = 200;
		public const int TangerineHueHcj = 1800;
		public const int FulvousHueHcj = 1900;
		public const int AmberHueHcj = 1950;
		public const int ForestHueHcj = 3800;
		public const int EmeraldHueHcj = 4300;
		public const int ViridianHueHcj = 5000;
		public const int TurquoiseHueHcj = 6000;
		public const int AzureHueHcj = 6700;
		public const int SapphireHueHcj = 7100;
		public const int LavenderHueHcj = 7900;
		public const int PurpleHueHcj = 8200;
		public const int FuchsiaHueHcj = 8500;

		public static void ApplyToneHue1EndShift( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl, int? toneHueEndShiftHcj )
		{
			if( toneHueEndShiftHsl != null || toneHueEndShiftHcj != null )
				configuration.ToneHue1EndShift = colorModel.SelectForColorModel( toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void ApplyToneHue2EndShift( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl, int? toneHueEndShiftHcj )
		{
			if( toneHueEndShiftHsl != null || toneHueEndShiftHcj != null )
				configuration.ToneHue2EndShift = colorModel.SelectForColorModel( toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void PassionTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( PassionHueHsl, PassionHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void RubyTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( RubyHueHsl, RubyHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void VermilionTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( VermilionHueHsl, VermilionHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void VermilionTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( VermilionHueHsl, VermilionHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void CoralTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( CoralHueHsl, CoralHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void TawnyTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( TawnyHueHsl, TawnyHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void TawnyTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( TawnyHueHsl, TawnyHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void CaramelTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( CaramelHueHsl, CaramelHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void CaramelTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( CaramelHueHsl, CaramelHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void MellowCaramelTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( MellowCaramelHueHsl, MellowCaramelHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, MellowCaramelHueShiftHsl, MellowCaramelHueShiftHcj );
		}

		public static void MellowCaramelTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( MellowCaramelHueHsl, MellowCaramelHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, MellowCaramelHueShiftHsl, MellowCaramelHueShiftHcj );
		}

		public static void TangerineTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( TangerineHueHsl, TangerineHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void FulvousTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( FulvousHueHsl, FulvousHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void FulvousTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( FulvousHueHsl, FulvousHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void AmberTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( AmberHueHsl, AmberHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void AmberTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( AmberHueHsl, AmberHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void ForestTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( ForestHueHsl, ForestHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void EmeraldTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( EmeraldHueHsl, EmeraldHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void EmeraldTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( EmeraldHueHsl, EmeraldHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void ViridianTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( ViridianHueHsl, ViridianHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void TurquoiseTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( TurquoiseHueHsl, TurquoiseHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void AzureTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( AzureHueHsl, AzureHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void SapphireTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( SapphireHueHsl, SapphireHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void SapphireTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( SapphireHueHsl, SapphireHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void LavenderTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( LavenderHueHsl, LavenderHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void LavenderTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( LavenderHueHsl, LavenderHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void PurpleTone1( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue1 = colorModel.SelectForColorModel( PurpleHueHsl, PurpleHueHcj );
			configuration.ApplyToneHue1EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void PurpleTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( PurpleHueHsl, PurpleHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}

		public static void FuchsiaTone2( this IBitonalGradientPaletteConfiguration configuration, ColorModelName colorModel
			, int? toneHueEndShiftHsl = null, int? toneHueEndShiftHcj = null )
		{
			configuration.ToneHue2 = colorModel.SelectForColorModel( FuchsiaHueHsl, FuchsiaHueHcj );
			configuration.ApplyToneHue2EndShift( colorModel, toneHueEndShiftHsl, toneHueEndShiftHcj );
		}
	}
}
