///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;
using Fluogen.Dependz.MdnReducers;
using Got.Libraries;

namespace Fluogen.DeployedScripts.MdnReducers
{
	public class HarshPowEvEvpEtAccumulator : MdnReducerWithAccumulator<DummyScriptConfiguration>
	{
		public override string Description => "Accumulate: a + 1 - (ev - evp) ^ s1 / et ^ s2";

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			if( fc.MdnReducerS1.IsZeroWithinEps() )
				throw new ArgumentException( $"{nameof( HarshPowEvEvpEtAccumulator )}: MdnReducerS1 == 0" );
			if( fc.MdnReducerS2.IsZeroWithinEps() )
				throw new ArgumentException( $"{nameof( HarshPowEvEvpEtAccumulator )}: MdnReducerS2 == 0" );
		}

		public override ColorDeterminant Accumulate( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, Mdn x, Mdn y, Mdn yp
			, double ev, double evp, ColorDeterminant a, int ri )
		{
			double et = fc.EscapeThreshold;

			return a.Add( 1 - (ev - evp).PowA( fc.MdnReducerS1 ) / et.PowA( fc.MdnReducerS2 ) );
		}
	}
}
