///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using ColorManagement.ColorModels;
using ColorManagement.Conversions;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	public class TestHslLightness : Palette<DummyScriptConfiguration>
	{
		public override int FullSize => 1 << 16;
		public override int QuickSize => 256;

		protected int H { get; } = 500;
		protected int S { get; } = 8000;
		protected int L { get; } = 10000;

		protected override void InitializeCache( IFluonConfiguration fc )
		{
		}

		protected override PixelRgbColor ComputeColor( IFluonConfiguration fc, double scalar )
		{
			int toneHue = fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1 != null
				? fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1.Value : H;

			double max = TypeableColorConverter.IntegerMax;
			double h = toneHue / max;
			double s = S / max;
			double l = scalar * L / max;

			var hsl = new Hsl( h, s, l );

			var rgb = HslConverter.ToRgb( hsl );

			var rgb16Bits = Rgb16Bits.FromRgbScalar( rgb );

			return PixelRgbColor.FromRgb16Bits( rgb16Bits.R, rgb16Bits.G, rgb16Bits.B );
		}
	}
}
